﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.NetworkInformation;
using Microsoft.Research.DynamicDataDisplay.Common;
using Reference = Microsoft.Research.DynamicDataDisplay.Common.WeakReference<System.Net.NetworkInformation.NetworkAvailabilityChangedEventHandler>;
using System.Windows.Threading;

namespace Microsoft.Research.DynamicDataDisplay.Maps.Servers.Network
{
	public class MyNetworkAvailabilityManager
	{
		private MyNetworkAvailabilityManager()
		{
			NetworkChange.NetworkAvailabilityChanged += NetworkChange_NetworkAvailabilityChanged;
			timer.Tick += new EventHandler(timer_Tick);
			timer.Start();
		}

		private void timer_Tick(object sender, EventArgs e)
		{
			int count = handlers.Count(r => r.IsAlive);
			if (count != handlers.Count)
			{
				Version++;
				handlers = handlers.Where(r => r.IsAlive).ToList();
			}
		}

		private void NetworkChange_NetworkAvailabilityChanged(object sender, NetworkAvailabilityEventArgs e)
		{
			var aliveRefs = handlers.Where(r => r.IsAlive).ToList();
			foreach (var r in aliveRefs)
			{
				var handler = r.Target;
				handler(sender, e);
			}
			handlers = aliveRefs;
		}

		private static readonly MyNetworkAvailabilityManager current = new MyNetworkAvailabilityManager();
		public static MyNetworkAvailabilityManager Current
		{
			get { return MyNetworkAvailabilityManager.current; }
		}

		private DispatcherTimer timer = new DispatcherTimer { Interval = TimeSpan.FromSeconds(100) };

		private List<Reference> handlers = new List<Reference>();

		private int version = 0;
		private int Version
		{
			get { return version; }
			set
			{
				version = value;
				//if (!timer.IsEnabled)
				//{
				//    timer.Start();
				//}
			}
		}

		public event NetworkAvailabilityChangedEventHandler NetworkAvailabilityChanged
		{
			add
			{
				handlers.Add(new Reference(value));
				Version++;
			}
			remove { }
		}
	}
}
