﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace Microsoft.Research.DynamicDataDisplay.Charts
{
	public sealed class LegendItemFactoryStore
	{
		private LegendItemFactoryStore()
		{
		}

		private static readonly LegendItemFactoryStore current = new LegendItemFactoryStore();
		public static LegendItemFactoryStore Current
		{
			get { return current; }
		}

		private readonly ObservableCollection<LegendItemFactory> factories = new ObservableCollection<LegendItemFactory>();
		public ObservableCollection<LegendItemFactory> Factories
		{
			get { return factories; }
		}

		public void RegisterFactory(LegendItemFactory factory)
		{
			if (factory == null)
				throw new ArgumentNullException("factory");

			factories.Add(factory);
		}

		public void BuildLegendItem(object chart, NewLegend legend)
		{
			foreach (var factory in factories)
			{
				var result = factory.TryBuildItem(chart, legend);
				if (result != null)
				{
					legendManagers.Add(chart, result);
					break;
				}
			}
		}

		private readonly Dictionary<object, IDisposable> legendManagers = new Dictionary<object, IDisposable>();
		public Dictionary<object, IDisposable> LegendManagers
		{
			get { return legendManagers; }
		} 

	}
}
